from __future__ import annotations

import logging

from core.constants import CellValue, UnitProperty
from core.logic import DistanceMap
from core.logic.Command import Command, STATE_PRIORITY
from core.logic.Logic import Logic


class NextTurnCommand(Command):

    def priority(self) -> int:
        return STATE_PRIORITY

    def check(self, logic: Logic) -> bool:
        state = logic.state
        if state.playerId not in state.playerIds:
            logging.error("Current player id not in state player list!")
            return False
        return True

    def execute(self, logic: Logic):
        state = logic.state

        # Next player id / turn
        playerId = state.playerId
        playerIds = state.playerIds
        playerIndex = playerIds.index(playerId) + 1
        if playerIndex >= len(playerIds):
            playerIndex = 0
            state.turn = state.turn + 1

            # Reset unit points
            units = state.world.units.units
            for unit in units.values():
                if unit.hasProperty(UnitProperty.MOVE_POINTS):
                    points = unit.getProperty(UnitProperty.MAX_MOVE_POINTS)
                    unit.setProperty(UnitProperty.MOVE_POINTS, points)
                if unit.hasProperty(UnitProperty.ACTION_POINTS):
                    points = unit.getProperty(UnitProperty.MAX_ACTION_POINTS)
                    unit.setProperty(UnitProperty.ACTION_POINTS, points)

        state.playerId = playerIds[playerIndex]


        # Next turn
        state.notifyTurnChanged()

